@font-face{font-family:"Recursive";font-style:normal;font-display:swap;font-weight:400;src:url(/static/_next/static/media/recursive-cyrillic-ext-400-normal.040a31ff.woff2) format("woff2"),url(/static/_next/static/media/recursive-all-400-normal.f02ba57e.woff) format("woff");unicode-range:U+0460-052f,U+1c80-1c88,U+20b4,U+2de0-2dff,U+a640-a69f,U+fe2e-fe2f}@font-face{font-family:"Recursive";font-style:normal;font-display:swap;font-weight:400;src:url(/static/_next/static/media/recursive-vietnamese-400-normal.c7aaa3fb.woff2) format("woff2"),url(/static/_next/static/media/recursive-all-400-normal.f02ba57e.woff) format("woff");unicode-range:U+0102-0103,U+0110-0111,U+0128-0129,U+0168-0169,U+01a0-01a1,U+01af-01b0,U+1ea0-1ef9,U+20ab}@font-face{font-family:"Recursive";font-style:normal;font-display:swap;font-weight:400;src:url(/static/_next/static/media/recursive-latin-ext-400-normal.46c8869d.woff2) format("woff2"),url(/static/_next/static/media/recursive-all-400-normal.f02ba57e.woff) format("woff");unicode-range:U+0100-024f,U+0259,U+1e??,U+2020,U+20a0-20ab,U+20ad-20cf,U+2113,U+2c60-2c7f,U+a720-a7ff}@font-face{font-family:"Recursive";font-style:normal;font-display:swap;font-weight:400;src:url(/static/_next/static/media/recursive-latin-400-normal.ad5f3e31.woff2) format("woff2"),url(/static/_next/static/media/recursive-all-400-normal.f02ba57e.woff) format("woff");unicode-range:U+00??,U+0131,U+0152-0153,U+02bb-02bc,U+02c6,U+02da,U+02dc,U+2000-206f,U+2074,U+20ac,U+2122,U+2191,U+2193,U+2212,U+2215,U+feff,U+fffd}body,html{padding:0;margin:0;font-family:-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Oxygen,Ubuntu,Cantarell,Fira Sans,Droid Sans,Helvetica Neue,sans-serif}a{color:inherit;text-decoration:none}*{box-sizing:border-box}.tldraw{position:fixed;top:0;left:0;right:0;bottom:0;width:100%;height:100%}
/*# sourceMappingURL=fb13a1146f6a82a5.css.map*/