(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[888],{6570:function(e,t,n){(window.__NEXT_P=window.__NEXT_P||[]).push(["/_app",function(){return n(8473)}])},8473:function(e,t,n){"use strict";n.r(t);var a=n(2322),o=(n(6937),n(7729)),r=n.n(o);n(981);function i(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function c(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{},a=Object.keys(n);"function"===typeof Object.getOwnPropertySymbols&&(a=a.concat(Object.getOwnPropertySymbols(n).filter((function(e){return Object.getOwnPropertyDescriptor(n,e).enumerable})))),a.forEach((function(t){i(e,t,n[t])}))}return e}var s="tldream",m="A tiny little diffusion drawing app.",l="https://github.com/Sanster/tldream",p="https://github.com/Sanster/tldream-frontend/blob/tldream/assets/tldream.png";t.default=function(e){var t=e.Component,n=e.pageProps;return(0,a.jsxs)(a.Fragment,{children:[(0,a.jsxs)(r(),{children:[(0,a.jsx)("meta",{name:"application-name",content:s}),(0,a.jsx)("meta",{name:"apple-mobile-web-app-capable",content:"yes"}),(0,a.jsx)("meta",{name:"apple-mobile-web-app-status-bar-style",content:"black"}),(0,a.jsx)("meta",{name:"apple-mobile-web-app-title",content:s}),(0,a.jsx)("meta",{name:"description",content:m}),(0,a.jsx)("meta",{name:"format-detection",content:"telephone=no"}),(0,a.jsx)("meta",{name:"mobile-web-app-capable",content:"yes"}),(0,a.jsx)("meta",{name:"theme-color",content:"#fafafa"}),(0,a.jsx)("meta",{name:"twitter:url",content:l}),(0,a.jsx)("meta",{name:"twitter:title",content:s}),(0,a.jsx)("meta",{name:"twitter:description",content:m}),(0,a.jsx)("meta",{name:"twitter:card",content:"summary_large_image"}),(0,a.jsx)("meta",{name:"twitter:creator",content:"@tldream"}),(0,a.jsx)("meta",{name:"twitter:site",content:"@tldream"}),(0,a.jsx)("meta",{name:"twitter:image",content:p}),(0,a.jsx)("meta",{property:"og:type",content:"website"}),(0,a.jsx)("meta",{property:"og:title",content:s}),(0,a.jsx)("meta",{property:"og:description",content:m}),(0,a.jsx)("meta",{property:"og:site_name",content:s}),(0,a.jsx)("meta",{property:"og:url",content:l}),(0,a.jsx)("meta",{property:"og:image",content:p}),(0,a.jsx)("meta",{name:"viewport",content:"width=device-width, initial-scale=1.0, maximum-scale=1.0, minimum-scale=1.0, user-scalable=no"}),(0,a.jsx)("link",{rel:"manifest",href:"/manifest.json"}),(0,a.jsx)("link",{rel:"shortcut icon",href:"/favicon.ico"}),(0,a.jsx)("link",{rel:"apple-touch-icon",sizes:"180x180",href:"/icons/apple-touch-icon.png"}),(0,a.jsx)("title",{children:"tldream"})]}),(0,a.jsx)(t,c({},n))]})}},6937:function(){},981:function(){},7729:function(e,t,n){e.exports=n(7016)}},function(e){var t=function(t){return e(e.s=t)};e.O(0,[774,179],(function(){return t(6570),t(2203)}));var n=e.O();_N_E=n}]);
//# sourceMappingURL=_app-86df0457e8e08c70.js.map